@@ -6,7 +6,7 @@
  *  these oopses for good, so we run those stupid tests... */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "exams.h"
@@ -437,6 +437,33 @@ static unsigned exam_paranoia16()
 	return result;
 }
 
+// Bug in reposition_dummy_indices() could result in correct expression
+// turned into one with inconsistent indices. Fixed on Aug 29, 2006
+static unsigned exam_paranoia17()
+{
+	varidx mu1(symbol("mu1"), 4);
+	varidx mu2(symbol("mu2"), 4);
+	varidx mu3(symbol("mu3"), 4);
+	varidx mu4(symbol("mu4"), 4);
+	varidx mu5(symbol("mu5"), 4);
+	varidx mu6(symbol("mu6"), 4);
+
+	exvector ev2;
+	ev2.push_back(mu3.toggle_variance());
+	ev2.push_back(mu6);
+	ev2.push_back(mu5.toggle_variance());
+	ev2.push_back(mu6.toggle_variance());
+	ev2.push_back(mu5);
+	ev2.push_back(mu3); 
+	// notice: all indices are contracted ...
+
+	ex test_cycl = indexed(symbol("A"), sy_cycl(), ev2);
+	test_cycl = test_cycl.simplify_indexed();
+	// ... so there should be zero free indices in the end.
+	return test_cycl.get_free_indices().size();
+}
+
+
 unsigned exam_paranoia()
 {
 	unsigned result = 0;
@@ -460,6 +487,7 @@ unsigned exam_paranoia()
 	result += exam_paranoia14();  cout << '.' << flush;
 	result += exam_paranoia15();  cout << '.' << flush;
 	result += exam_paranoia16();  cout << '.' << flush;
+	result += exam_paranoia17();  cout << '.' << flush;
 	
 	if (!result) {
 		cout << " passed " << endl;
