@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's symbolic  integral. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -279,7 +279,7 @@ ex adaptivesimpson(const ex & x, const ex & a_in, const ex & b_in, const ex & f,
 	fbvec[i] = subsvalue(x, b, f);
 	svec[i] = hvec[i]*(favec[i]+4*fcvec[i]+fbvec[i])/3;
 	lvec[i] = 1;
-	errorvec[i] = error*svec[i];
+	errorvec[i] = error*abs(svec[i]);
 
 	while (i>0) {
 		ex fd = subsvalue(x, avec[i]+hvec[i]/2, f);
@@ -292,7 +292,7 @@ ex adaptivesimpson(const ex & x, const ex & a_in, const ex & b_in, const ex & f,
 		ex nu4 = fbvec[i];
 		ex nu5 = hvec[i];
 		// hopefully prevents a crash if the function is zero sometimes.
-		ex nu6 = max(errorvec[i], (s1+s2)*error);
+		ex nu6 = max(errorvec[i], abs(s1+s2)*error);
 		ex nu7 = svec[i];
 		int nu8 = lvec[i];
 		--i;
