@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's special tensors. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -355,6 +355,8 @@ bool tensor::replace_contr_index(exvector::iterator self, exvector::iterator oth
 again:
 	if (self_idx->is_symbolic()) {
 		for (size_t i=1; i<other->nops(); i++) {
+			if (! is_a<idx>(other->op(i)))
+				continue;
 			const idx &other_idx = ex_to<idx>(other->op(i));
 			if (is_dummy_pair(*self_idx, other_idx)) {
 
